# This file is part of the eseas project
# Copyright (C) 2024 Sermet Pekin
#
# This source code is free software; you can redistribute it and/or
# modify it under the terms of the European Union Public License
# (EUPL), Version 1.2, as published by the European Commission.
#
# You should have received a copy of the EUPL version 1.2 along with this
# program. If not, you can obtain it at:
# <https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12>.
#
# This source code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# European Union Public License for more details.
#
# Alternatively, if agreed upon, you may use this code under any later
# version of the EUPL published by the European Commission.


# ====================================================================
#
# eseas
#
from .cruncher_classes import Cruncher
from .folder_class import (
    DemetraFolder,
    JavaBinFolder,
    WorkspaceFolder,
    CruncherFolder,
)

# ====================================================================


class SingleOptions:
    """SingleOptions"""

    def __new__(cls):
        if not hasattr(cls, "instance"):
            cls.instance = super(SingleOptions, cls).__new__(cls)
        return cls.instance

    def set_items(cls, options):
        cls.instance.options = options


class SeasonalOptions:
    def __init__(
        self,
        demetra_folder=None,
        java_folder=None,
        local_folder=None,
        test=False,
        verbose=False,
        replace_original_files=False,
        auto_approve=False,
        result_file_names=(
            "sa",
            "s",
            "cal",
        ),
        workspace_mode=True,
        file_name_explanation=True,
        java_bin=None,
        auto_download=False,
        replace_general_params=False,
        csvlayout="vtable",
    ):
        try:
            self._init_impl(
                demetra_folder=demetra_folder,
                java_folder=java_folder,
                local_folder=local_folder,
                test=test,
                verbose=verbose,
                replace_original_files=replace_original_files,
                auto_approve=auto_approve,
                result_file_names=result_file_names,
                workspace_mode=workspace_mode,
                file_name_explanation=file_name_explanation,
                java_bin=java_bin,
                auto_download=auto_download,
                replace_general_params=replace_general_params,
                csvlayout=csvlayout,
            )
        except Exception as e:
            from .error_logger import log_eseas_error
            import traceback
            error_msg = f"{str(e)}\n{traceback.format_exc()}"
            # We don't have the fully constructed options object, but we can pass the locals
            opts_dict = {
                "demetra_folder": demetra_folder,
                "java_folder": java_folder,
                "local_folder": local_folder,
                "test": test,
                "verbose": verbose,
                "replace_original_files": replace_original_files,
                "auto_approve": auto_approve,
                "result_file_names": result_file_names,
                "workspace_mode": workspace_mode,
                "file_name_explanation": file_name_explanation,
                "java_bin": java_bin,
                "auto_download": auto_download,
                "replace_general_params": replace_general_params,
                "csvlayout": csvlayout,
            }
            class OptsWrapper:
                def __init__(self, d):
                    for k, v in d.items():
                        setattr(self, k, v)
            log_eseas_error(error_msg, OptsWrapper(opts_dict))
            raise
        else:
            # Also set up the .eseas directory to provide examples even if no crash occurs
            from .error_logger import setup_eseas_logger
            try:
                setup_eseas_logger()
            except Exception:
                pass

    def _init_impl(
        self,
        demetra_folder=None,
        java_folder=None,
        local_folder=None,
        test=False,
        verbose=False,
        replace_original_files=False,
        auto_approve=False,
        result_file_names=(
            "sa",
            "s",
            "cal",
        ),
        workspace_mode=True,
        file_name_explanation=True,
        java_bin=None,
        auto_download=False,
        replace_general_params=False,
        csvlayout="list",
    ):
        
        # If auto_download is True, automatically fetch the cruncher directly to java_folder
        if auto_download:
            from .download_tools import download_jwsacruncher
            import pathlib
            target_path = pathlib.Path(java_folder) if java_folder else pathlib.Path.home() / ".eseas"
            java_folder = str(download_jwsacruncher(target_path))

        self.demetra_folder = DemetraFolder(demetra_folder)
        self.java_folder = CruncherFolder(java_folder)
        self.local_folder = WorkspaceFolder(local_folder)
        self.java_bin = JavaBinFolder(java_bin)

        self.demetra_folder = str(self.demetra_folder)
        self.java_folder = str(self.java_folder)
        self.local_folder = str(self.local_folder)
        self.java_bin = java_bin if java_bin else None

        self.test = test
        self.verbose = verbose
        self.replace_original_files = replace_original_files
        self.auto_approve = auto_approve
        self.result_file_names = result_file_names
        self.workspace_mode = workspace_mode
        self.file_name_explanation = file_name_explanation
        self.replace_general_params = replace_general_params
        self.csvlayout = csvlayout

        self.set_options(workspace_mode)
        so = SingleOptions()
        so.set_items(self)

    def __repr__(self):
        template = f"""
    options= SeasonalOptions(
            demetra_folder="{self.demetra_folder}",
            java_folder="{self.java_folder}",
            local_folder="{self.local_folder}",
            test={self.test},
            verbose= {self.verbose},
            replace_original_files={self.replace_original_files},
            auto_approve={self.auto_approve},
            result_file_names={self.result_file_names},
            workspace_mode={self.workspace_mode},
            java_bin = {self.java_bin},
            replace_general_params={self.replace_general_params},
            csvlayout="{self.csvlayout}"
    )
        """
        return template

    def set_options(self, workspace_mode):
        c = Cruncher()
        c.set_items(
            local_work_space=self.local_folder,
            crunch_folder=self.java_folder,
            demetra_folder=self.demetra_folder,
            workspace_mode=workspace_mode,
            file_name_explanation=self.file_name_explanation,
            java_bin=self.java_bin,
        )



import pytest
from unittest.mock import patch, MagicMock
from eseas.core.seasonal_options import SeasonalOptions as Options
from eseas.core.seasonal_general import SeasonalADV as Seasonal
from pathlib import Path

def test_auto_download_enabled(tmp_path):
    """Test if auto_download=True properly invokes the download function."""
    with patch("eseas.core.download_tools.download_jwsacruncher") as mock_download:
        with patch("eseas.core.cruncher_classes.check_cruncher", return_value=True):
            fake_bin_path = tmp_path / "mock_bin"
            mock_download.return_value = fake_bin_path
            
            opts = Options(
                demetra_folder="fake_source",
                java_folder=str(tmp_path),
                local_folder="fake_local",
                auto_download=True
            )
            
            # Verify the downloader was called
            mock_download.assert_called_once()
            
            # Verify the options java_folder has been updated with the result
            assert str(opts.java_folder) == str(fake_bin_path)

def test_auto_download_disabled(tmp_path):
    """Test if auto_download=False skips the download function."""
    with patch("eseas.core.download_tools.download_jwsacruncher") as mock_download:
        with patch("eseas.core.cruncher_classes.check_cruncher", return_value=True):
            opts = Options(
                demetra_folder="fake_source",
                java_folder=str(tmp_path),
                local_folder="fake_local",
                auto_download=False
            )
            
            # Verify the downloader was NOT called
            mock_download.assert_not_called()
            assert str(opts.java_folder) == str(tmp_path)

@patch("eseas.core.seasonal_general.get_input_from_user")
@patch("eseas.core.seasonal_general.SeasonalADV.part2")
@patch("eseas.core.seasonal_general.run_bat_commands")
@patch("eseas.core.seasonal_general.get_demetra_files")
@patch("eseas.core.seasonal_general.write_bat_file_demetra")
def test_auto_approve_true(mock_write_bat, mock_get_demetra, mock_run_bat_commands, mock_part2, mock_get_input, tmp_path):
    """Test if auto_approve bypasses the get_input_from_user prompt."""
    with patch("eseas.core.cruncher_classes.check_cruncher", return_value=True):
        opts = Options(
            demetra_folder="fake_source",
            java_folder=str(tmp_path),
            local_folder="fake_local",
            auto_approve=True,
        )
        
        m = Seasonal(opts)
        
        try:
            m.part1()
        except Exception:
            pass
            
        mock_get_input.assert_not_called()

@patch("eseas.core.seasonal_general.get_input_from_user")
@patch("eseas.core.seasonal_general.SeasonalADV.part2")
@patch("eseas.core.seasonal_general.run_bat_commands")
@patch("eseas.core.seasonal_general.get_demetra_files")
@patch("eseas.core.seasonal_general.write_bat_file_demetra")
def test_auto_approve_false(mock_write_bat, mock_get_demetra, mock_run_bat_commands, mock_part2, mock_get_input, tmp_path):
    """Test if auto_approve=False correctly calls the get_input_from_user prompt."""
    with patch("eseas.core.cruncher_classes.check_cruncher", return_value=True):
        opts = Options(
            demetra_folder="fake_source",
            java_folder=str(tmp_path),
            local_folder="fake_local",
            auto_approve=False,
        )
        
        mock_get_input.return_value = True # Simulate user pressing "Y"
        mock_get_demetra.return_value = []
        m = Seasonal(opts)
        
        try:
            m.part1()
        except Exception as e:
            print(f"Exception happened: {e}")
            
        mock_get_input.assert_called_once()



# This file is part of the eseas project
# Copyright (C) 2024 Sermet Pekin
#
# This source code is free software; you can redistribute it and/or
# modify it under the terms of the European Union Public License
# (EUPL), Version 1.2, as published by the European Commission.
#
# You should have received a copy of the EUPL version 1.2 along with this
# program. If not, you can obtain it at:
# <https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12>.
#
# This source code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# European Union Public License for more details.
#
# Alternatively, if agreed upon, you may use this code under any later
# version of the EUPL published by the European Commission.


import pytest
from pathlib import Path
from eseas import Seasonal
from eseas import Options
from eseas.core.seas_testing_utils import get_testing_utils
from eseas.tests.test_utils import skip_if_no_cruncher


testing_utils = get_testing_utils()
demetra_folder = testing_utils.demetra_folder
java_folder = testing_utils.java_folder
local_folder = testing_utils.local_folder


@skip_if_no_cruncher
def test_multivariate_monthly(tmp_path):
    """Test multivariate workspace with monthly data (4 series)"""

    # Create a temporary output folder
    temp_local_folder = tmp_path / "multivariate_output"
    temp_local_folder.mkdir()

    # Set up options pointing to the multivariate workspace
    workspace_path = Path(demetra_folder) / "multivariate.xml"
    assert workspace_path.exists(), f"Multivariate workspace not found at {workspace_path}"

    options = Options(
        demetra_folder=str(workspace_path.parent),
        java_folder=java_folder,
        local_folder=str(temp_local_folder),
        test=False,
        verbose=True,
        replace_original_files=False,
        result_file_names=("sa", "s", "cal"),
        workspace_mode=True,
        java_bin=None,
        replace_general_params=True,
        auto_approve=True,
        csvlayout="list"
    )

    m = Seasonal(options)
    m.run()

    # Verify output was generated
    # The structure should be: temp_local_folder/@eseas_wspace/test_output/multivariate/SAProcessing-1/
    output_dir = temp_local_folder / "@eseas_wspace" / "test_output" / "multivariate" / "SAProcessing-1"

    # Check that output directory exists
    assert output_dir.exists(), f"Output directory not created at {output_dir}"

    # Check that CSV files were generated
    csv_files = list(output_dir.glob("*.csv"))
    assert len(csv_files) > 0, "No CSV files generated"

    # Should have a reasonable number of output files for 2 series
    # Each series generates multiple CSV files, so we expect at least 20 files total
    assert len(csv_files) >= 20, f"Expected at least 20 CSV files for 2 series, got {len(csv_files)}"


@skip_if_no_cruncher
def test_multivariate_quarterly(tmp_path):
    """Test multivariate workspace with quarterly data (3 series)"""

    # Create a temporary output folder
    temp_local_folder = tmp_path / "multivariate_quarterly_output"
    temp_local_folder.mkdir()

    # Set up options pointing to the multivariate_quarterly workspace
    workspace_path = Path(demetra_folder) / "multivariate_quarterly.xml"
    assert workspace_path.exists(), f"Multivariate quarterly workspace not found at {workspace_path}"

    options = Options(
        demetra_folder=str(workspace_path.parent),
        java_folder=java_folder,
        local_folder=str(temp_local_folder),
        test=False,
        verbose=True,
        replace_original_files=False,
        result_file_names=("sa", "s", "cal"),
        workspace_mode=True,
        java_bin=None,
        replace_general_params=True,
        auto_approve=True,
        csvlayout="list"
    )

    m = Seasonal(options)
    m.run()

    # Verify output was generated
    output_dir = temp_local_folder / "@eseas_wspace" / "test_output" / "multivariate_quarterly" / "SAProcessing-1"

    # Check that output directory exists
    assert output_dir.exists(), f"Output directory not created at {output_dir}"

    # Check that CSV files were generated
    csv_files = list(output_dir.glob("*.csv"))
    assert len(csv_files) > 0, "No CSV files generated"

    # Should have a reasonable number of output files for 2 series
    # Each series generates multiple CSV files, so we expect at least 20 files total
    assert len(csv_files) >= 20, f"Expected at least 20 CSV files for 2 series, got {len(csv_files)}"


@skip_if_no_cruncher
def test_multivariate_combined_output(tmp_path):
    """Test that combined output is properly generated for multivariate data"""

    temp_local_folder = tmp_path / "multivariate_combined"
    temp_local_folder.mkdir()

    workspace_path = Path(demetra_folder) / "multivariate.xml"

    options = Options(
        demetra_folder=str(workspace_path.parent),
        java_folder=java_folder,
        local_folder=str(temp_local_folder),
        test=False,
        verbose=False,
        replace_original_files=False,
        result_file_names=("sa", "s"),
        workspace_mode=True,
        java_bin=None,
        replace_general_params=True,
        auto_approve=True,
        csvlayout="vtable"
    )

    m = Seasonal(options)
    m.run()

    # Check for combined.xlsx output
    expected_output = temp_local_folder / "test_output" / "multivariate" / "combined.xlsx"
    assert expected_output.exists(), "Combined output file was not generated"

    # Verify the combined file has data
    import pandas as pd
    df = pd.read_excel(expected_output)
    assert len(df) > 0, "Combined output file is empty"

    # Should have data for multiple series
    assert len(df) > 50, f"Expected substantial data in combined output, got {len(df)} rows"



# This file is part of the eseas project
# Copyright (C) 2024 Sermet Pekin
#
# This source code is free software; you can redistribute it and/or
# modify it under the terms of the European Union Public License
# (EUPL), Version 1.2, as published by the European Commission.
#
# You should have received a copy of the EUPL version 1.2 along with this
# program. If not, you can obtain it at:
# <https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12>.
#
# This source code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# European Union Public License for more details.
#
# Alternatively, if agreed upon, you may use this code under any later
# version of the EUPL published by the European Commission.

from dataclasses import dataclass

from .cruncher_classes import get_cruncher

from abc import ABC, abstractmethod
from ._options import demetra_command_file_name


@dataclass
class DemetraCaller(ABC):
    @property
    def cf(self):
        return get_cruncher().crunch_folder

    @property
    def ws(self):
        return get_cruncher().local_work_space

    @abstractmethod
    def cruncher_command(self): ...

    @abstractmethod
    def demetra_command_file_name(self): ...

    @abstractmethod
    def exec_file_name(self, file_name): ...


class DemetraCallerWindows(DemetraCaller):
    def cruncher_command(self):
        return rf'call "{self.cf}/jwsacruncher.bat"'

    def demetra_command_file_name(self):
        return rf"{self.ws}/{demetra_command_file_name}.bat"

    def exec_file_name(self, file_name):
        return rf"{self.ws}/{file_name}.bat"


@dataclass
class DemetraCallerLinux(DemetraCaller):
    def cruncher_command(self):
        return rf'"{self.cf}/jwsacruncher"'

    def demetra_command_file_name(self):
        return rf"{self.ws}/{demetra_command_file_name}.sh"

    def exec_file_name(self, file_name):
        return rf"{self.ws}/{file_name}.sh"


@dataclass
class DemetraCallerMac(DemetraCallerLinux): ...



# This file is part of the eseas project
# Copyright (C) 2024 Sermet Pekin
#
# This source code is free software; you can redistribute it and/or
# modify it under the terms of the European Union Public License
# (EUPL), Version 1.2, as published by the European Commission.
#
# You should have received a copy of the EUPL version 1.2 along with this
# program. If not, you can obtain it at:
# <https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12>.
#
# This source code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# European Union Public License for more details.
#
# Alternatively, if agreed upon, you may use this code under any later
# version of the EUPL published by the European Commission.


import pytest
from pathlib import Path
import json


def test_success_log_format():
    """Test that success logs contain comprehensive information"""
    log_file = Path.cwd() / ".eseas" / ".logs" / "last_good_run.log"

    if not log_file.exists():
        pytest.skip("No success log file exists yet")

    # Read the log file and extract JSON entries
    log_content = log_file.read_text(encoding="utf-8")

    # Split by the log timestamp pattern to get individual log entries
    import re
    entries = re.split(r'\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d{3} - \w+ - INFO - ', log_content)

    # Get the last non-empty entry
    last_json = None
    for entry in reversed(entries):
        entry = entry.strip()
        if entry and entry.startswith('{'):
            try:
                last_json = json.loads(entry)
                break
            except json.JSONDecodeError:
                continue

    if not last_json:
        pytest.skip("Could not parse log entry")

    # Verify comprehensive information is present
    assert "status" in last_json
    assert last_json["status"] == "success"
    assert "eseas_version" in last_json
    assert "timestamp" in last_json
    assert "execution_time_seconds" in last_json
    assert "system" in last_json
    assert "options" in last_json
    assert "paths" in last_json
    assert "processing_stats" in last_json

    # Verify system info
    system = last_json["system"]
    assert "platform" in system
    assert "python_version" in system
    assert "java_available" in system
    assert "java_version" in system

    # Verify processing stats
    stats = last_json["processing_stats"]
    assert "xml_files_found" in stats or "error" in stats

    print("\nSuccess log verification passed!")
    print(f"  - eseas version: {last_json['eseas_version']}")
    print(f"  - execution time: {last_json['execution_time_seconds']}s")
    print(f"  - platform: {system['platform']}")
    if "xml_files_found" in stats:
        print(f"  - XML files processed: {stats['xml_files_found']}")


def test_error_log_creation():
    """Test that error logs are created when exceptions occur"""
    from eseas import Options, Seasonal

    # Try to run with invalid folder to trigger error
    with pytest.raises(Exception):
        options = Options(
            demetra_folder="nonexistent_folder_xyz123",
            java_folder="/usr/bin",
            local_folder="./temp_test"
        )
        m = Seasonal(options)
        m.run()

    # Check if error log was created
    error_log = Path.cwd() / ".eseas" / ".logs" / "failed_runs.log"
    assert error_log.exists(), "Error log should be created when exception occurs"

    # Verify error log contains useful information
    log_content = error_log.read_text(encoding="utf-8")
    assert "error" in log_content.lower()

    print("\nError log creation verified!")


def test_log_directory_creation():
    """Test that log directory is always created"""
    log_dir = Path.cwd() / ".eseas" / ".logs"
    assert log_dir.exists(), "Log directory should be created"
    assert log_dir.is_dir(), "Log path should be a directory"

    # Check that example script was created
    example_script = Path.cwd() / ".eseas" / "quick_start_example.py"
    assert example_script.exists(), "Quick start example should be created"

    # Check that README was created
    readme = Path.cwd() / ".eseas" / "README.txt"
    assert readme.exists(), "README should be created"

    print("\nLog directory structure verified!")



from pathlib import Path
import os
import pandas as pd

from evdspy.EVDSlocal.common.file_classes import FileItem , file_items_update

# eseas
from eseas.core.seasonal_options import SeasonalOptions as Options
from eseas.core.seas_utils import get_xml_demetra
from eseas.core.cruncher_classes import get_cruncher


def make_float(d: pd.DataFrame):
    if "Unnamed: 0" in d.columns:
        cols = d.columns.drop("Unnamed: 0")
    else:
        cols = d.columns.to_list()[1:]
    d[cols] = d[cols].astype(str).apply(lambda x: x.str.replace(",", ".").astype(float))
    return d


class ResultCollector:
    """
    Collects seasonality result parts (sa, s, cal, etc.) and compiles them into Excel resources.
    """
    def __init__(
        self,
        options: Options,
        out_folder: str = None,
        out_file_name: str = "combined",
        encoding: str = "latin-1",
    ):
        self.options = options
        self.out_folder = out_folder
        self.out_file_name = out_file_name
        self.encoding = encoding
        self.parts = options.result_file_names

    def get_xml_folders(self, xml_folders: list = None) -> list:
        if xml_folders is not None:
            return xml_folders
        files: list[FileItem] = get_xml_demetra(self.options.demetra_folder)
        files = file_items_update(files)
        return [x.encoded_name for x in files ]


    def get_source_file(self, xml_folder: str, part: str) -> Path:
        return (
            Path(get_cruncher().local_work_space)
            / "test_output"
            / xml_folder
            / "SAProcessing-1"
            / f"series_{part}.csv"
        )

    def load_sheet(self, source_file: Path) -> pd.DataFrame:
        sheet = pd.read_csv(
            source_file,
            encoding=self.encoding,
            delimiter=";",
        )
        return make_float(sheet)

    def get_output_file_name(self, xml_folder: str) -> Path:
        if self.out_folder is None:
            out_file_name_full = (
                Path(self.options.local_folder)
                / "test_output"
                / xml_folder
                / f"{self.out_file_name}.xlsx"
            )
            os.makedirs(out_file_name_full.parent, exist_ok=True)
        else:
            dest_folder = Path(self.out_folder) / xml_folder
            os.makedirs(dest_folder, exist_ok=True)
            out_file_name_full = dest_folder / f"{self.out_file_name}.xlsx"
        return out_file_name_full

    def process_folder(self, xml_folder: str):
        # We explicitly store parts in a dictionary to prevent zip mismatch
        # in case a specific part fails to load.
        sheets_data = {}
        for part in self.parts:
            source_file = self.get_source_file(xml_folder, part)
            try:
                sheets_data[part] = self.load_sheet(source_file)
            except Exception:
                import traceback
                traceback.print_exc()
                print(f"passing collecting {part} from {source_file}")

        if not sheets_data:
            return

        out_file_name_full = self.get_output_file_name(xml_folder)
        with pd.ExcelWriter(out_file_name_full) as writer:
            for part, sheet in sheets_data.items():
                sheet.to_excel(writer, sheet_name=part)
        print(f"[created] {out_file_name_full}")

    def collect(self, xml_folders=None):
        folders = self.get_xml_folders(xml_folders)
        for xml_folder in folders:
            self.process_folder(xml_folder)


def collect_parts_of_results(
    options: Options,
    xml_folders=None,
    out_folder=None,
    out_file_name="combined",
    encoding="latin-1",
):
    """
    Collects parts of results using the ResultCollector class.
    """
    collector = ResultCollector(
        options=options,
        out_folder=out_folder,
        out_file_name=out_file_name,
        encoding=encoding,
    )
    collector.collect(xml_folders)



import os
import pytest
from pathlib import Path
import subprocess
from eseas.core.cruncher_classes import get_cruncher
from eseas.core.demetra_caller import DemetraCallerLinux, DemetraCallerWindows, DemetraCallerMac
from eseas import Options
from eseas.tests.test_utils import skip_if_no_cruncher

def test_executable_quoting(tmp_path):
    fake_java = tmp_path / "opt" / "my cool cruncher"
    fake_java.mkdir(parents=True)
    fake_in = tmp_path / "in"
    fake_in.mkdir(parents=True)

    opt = Options(
        demetra_folder=str(fake_in),
        java_folder=str(fake_java),
        local_folder=str(tmp_path / "out"),
        workspace_mode=False,
        test=True
    )
    linux_cmd = DemetraCallerLinux().cruncher_command()
    mac_cmd = DemetraCallerMac().cruncher_command()
    win_cmd = DemetraCallerWindows().cruncher_command()
    
    assert linux_cmd.startswith('"') and linux_cmd.endswith('"'), "Linux caller should wrap path in quotes"
    assert mac_cmd.startswith('"') and mac_cmd.endswith('"'), "Mac caller should wrap path in quotes"
    assert win_cmd.startswith('call "') and win_cmd.endswith('"'), "Windows caller should wrap call path in quotes"
    assert "my cool cruncher/jwsacruncher" in linux_cmd

def test_workspace_creation_with_spaces_and_special_chars(tmp_path):
    sp_dir = tmp_path / "my space folder!@#"
    sp_dir.mkdir()
    out_dir = tmp_path / "out location @#"
    
    fake_java = tmp_path / "opt" / "bin"
    fake_java.mkdir(parents=True)
    
    # Use workspace_mode=True to ensure @eseas_wspace builds correctly
    opt = Options(
        demetra_folder=str(sp_dir),
        java_folder=str(fake_java),
        local_folder=str(out_dir),
        workspace_mode=True,
        test=True
    )
    c = get_cruncher()
    assert "my space folder!@#" in str(c.demetra_folder)
    assert Path(c.local_work_space).exists(), "Local workspace directory not created"
    assert "@eseas_wspace" in str(c.local_work_space), "@eseas_wspace dir tracking missing"

@skip_if_no_cruncher
def test_hardcoded_jwsacruncher_direct_call():
    """
    Run jwsacruncher directly to see if the executable operates properly.
    Uses subprocess to bypass wrapper logic and catch CLI failures directly.
    """
    cruncher_dir = os.environ.get("JAVA_CRUNCHER_BIN")
    exe_name = "jwsacruncher.bat" if os.name == "nt" else "jwsacruncher"
    exe_path = Path(cruncher_dir) / exe_name
    
    assert exe_path.exists(), f"Expected executable {exe_path} to exist from test setup."
    
    try:
        # Run with '--version' or empty args to catch raw execution errors safely
        res = subprocess.run([str(exe_path)], capture_output=True, text=True)
        assert res.returncode in (0, 1, 2)
    except OSError as e:
        pytest.fail(f"jwsacruncher failed to execute directly via OS: {e}")

def test_xml_url_encoding(tmp_path):
    """
    Ensure the path injected into XML is correctly URL encoded as required by JDemetra's tsprovider.
    """
    from urllib.parse import quote
    
    complex_dir = tmp_path / "complex name with spaces"
    complex_dir.mkdir()
    
    excel_path = complex_dir / "data.xlsx"
    abs_path = str(excel_path.absolute())
    
    mac_url_part = quote(abs_path, safe="") 
    
    # Assert forward slashes are encoded to %2F (strict tsprovider requirement)
    assert "%2F" in mac_url_part
    assert "complex%20name%20with%20spaces" in mac_url_part
    assert "/" not in mac_url_part



#!/usr/bin/env python
"""
eseas CLI - Command line interface for eseas seasonal adjustment tool
"""

import sys
import argparse
from pathlib import Path
import yaml
from typing import Optional
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

console = Console()


DEFAULT_CONFIG = {
    "demetra_folder": None,  # Will be set from first run
    "java_folder": None,  # Will use auto-download
    "local_folder": "./eseas_output",
    "auto_download": True,
    "auto_approve": False,
    "csvlayout": "vtable",
    "workspace_mode": True,
    "result_file_names": ["sa", "s", "cal"],
    "verbose": False,
    "test": False,
    "replace_general_params": False,
    "replace_original_files": False,
    "file_name_explanation": True,
}


def get_config_path() -> Path:
    """Get the path to the config file in current directory"""
    return Path.cwd() / "eseas_config.yaml"


def create_default_config(workspace_path: Optional[str] = None) -> dict:
    """Create default configuration with optional workspace path"""
    config = DEFAULT_CONFIG.copy()
    if workspace_path:
        config["demetra_folder"] = str(Path(workspace_path).absolute())
    return config


def load_config(config_path: Path) -> dict:
    """Load configuration from YAML file"""
    try:
        with open(config_path, "r") as f:
            config = yaml.safe_load(f)
            if config is None:
                return {}
            return config
    except FileNotFoundError:
        return {}
    except yaml.YAMLError as e:
        console.print(f"[red]Error parsing config file: {e}[/red]")
        sys.exit(1)


def save_config(config: dict, config_path: Path):
    """Save configuration to YAML file"""
    try:
        with open(config_path, "w") as f:
            yaml.dump(config, f, default_flow_style=False, sort_keys=False)
        console.print(f"[green]✓[/green] Configuration saved to {config_path}")
    except Exception as e:
        console.print(f"[red]Error saving config: {e}[/red]")
        sys.exit(1)


def show_config(config: dict):
    """Display current configuration in a nice table"""
    table = Table(title="Current Configuration")
    table.add_column("Parameter", style="cyan", no_wrap=True)
    table.add_column("Value", style="green")

    for key, value in config.items():
        if isinstance(value, list):
            value_str = ", ".join(str(v) for v in value)
        else:
            value_str = str(value) if value is not None else "[dim]not set[/dim]"
        table.add_row(key, value_str)

    console.print(table)


def cmd_run(args):
    """Run seasonal adjustment"""
    from eseas import Options, Seasonal

    # Determine config file location
    if args.config:
        config_path = Path(args.config)
        if not config_path.exists():
            console.print(f"[red]Config file not found: {config_path}[/red]")
            sys.exit(1)
    else:
        config_path = get_config_path()

    # Load existing config or create new one
    if config_path.exists():
        console.print(f"[cyan]Loading configuration from {config_path}[/cyan]")
        config = load_config(config_path)
    else:
        # First run - create config
        workspace_path = args.workspace or args.demetra_folder
        if not workspace_path:
            console.print(
                "[yellow]No workspace path provided and no config file found.[/yellow]"
            )
            console.print("[cyan]Usage:[/cyan] eseas run <workspace_path>")
            console.print("[cyan]Or:[/cyan]     eseas run --workspace <path>")
            sys.exit(1)

        console.print("[cyan]First run detected! Creating configuration file...[/cyan]")
        config = create_default_config(workspace_path)
        save_config(config, config_path)
        console.print()
        console.print(
            Panel.fit(
                f"""[green]Configuration created![/green]

Edit [cyan]{config_path}[/cyan] to customize settings.
Next time, simply run: [yellow]eseas run[/yellow]""",
                title="Setup Complete",
                border_style="green",
            )
        )
        console.print()

    # Override config with command line arguments
    if args.workspace:
        config["demetra_folder"] = str(Path(args.workspace).absolute())
    if args.output:
        config["local_folder"] = str(Path(args.output).absolute())
    if args.verbose is not None:
        config["verbose"] = args.verbose
    if args.test is not None:
        config["test"] = args.test
    if args.csvlayout:
        config["csvlayout"] = args.csvlayout

    # Validate required fields
    if not config.get("demetra_folder"):
        console.print(
            "[red]Error: demetra_folder not set in config or command line[/red]"
        )
        sys.exit(1)

    # Create Options and run
    try:
        console.print("[cyan]Starting seasonal adjustment...[/cyan]")
        options = Options(
            demetra_folder=config["demetra_folder"],
            java_folder=config.get("java_folder"),
            local_folder=config["local_folder"],
            auto_download=config.get("auto_download", True),
            auto_approve=config.get("auto_approve", False),
            csvlayout=config.get("csvlayout", "vtable"),
            workspace_mode=config.get("workspace_mode", True),
            result_file_names=tuple(config.get("result_file_names", ["sa", "s", "cal"])),
            verbose=config.get("verbose", False),
            test=config.get("test", False),
            replace_general_params=config.get("replace_general_params", False),
            replace_original_files=config.get("replace_original_files", False),
            file_name_explanation=config.get("file_name_explanation", True),
        )

        m = Seasonal(options)
        m.run()

        console.print()
        console.print(Panel.fit(
            "[green]✓ Seasonal adjustment completed successfully![/green]",
            border_style="green",
        ))

    except Exception as e:
        console.print(f"\n[red]Error during execution: {e}[/red]")
        console.print("\n[yellow]Check logs at:[/yellow] .eseas/.logs/failed_runs.log")
        sys.exit(1)


def cmd_config(args):
    """Manage configuration"""
    config_path = Path(args.config) if args.config else get_config_path()

    if args.action == "show":
        # Show current configuration
        if not config_path.exists():
            console.print(f"[yellow]No configuration file found at {config_path}[/yellow]")
            console.print("[cyan]Run 'eseas run <workspace>' to create one.[/cyan]")
            return

        config = load_config(config_path)
        show_config(config)

    elif args.action == "set":
        # Set a configuration value
        if not args.key or not args.value:
            console.print("[red]Error: Both --key and --value are required[/red]")
            sys.exit(1)

        # Load existing or create new
        if config_path.exists():
            config = load_config(config_path)
        else:
            config = create_default_config()

        # Parse value (handle booleans and lists)
        value = args.value
        if value.lower() in ("true", "yes", "on"):
            value = True
        elif value.lower() in ("false", "no", "off"):
            value = False
        elif "," in value:
            value = [v.strip() for v in value.split(",")]

        config[args.key] = value
        save_config(config, config_path)
        console.print(f"[green]✓[/green] Set {args.key} = {value}")

    elif args.action == "init":
        # Create new config with defaults
        if config_path.exists() and not args.force:
            console.print(f"[yellow]Config file already exists: {config_path}[/yellow]")
            console.print("[cyan]Use --force to overwrite[/cyan]")
            return

        workspace = args.workspace if args.workspace else None
        config = create_default_config(workspace)
        save_config(config, config_path)
        console.print()
        show_config(config)


def cmd_validate(args):
    """Validate workspace and configuration"""
    from eseas.core.folder_class import FolderClass

    workspace_path = Path(args.workspace)

    console.print(f"[cyan]Validating workspace:[/cyan] {workspace_path}")
    console.print()

    # Check if path exists
    if not workspace_path.exists():
        console.print(f"[red]✗ Path does not exist: {workspace_path}[/red]")
        sys.exit(1)
    console.print(f"[green]✓[/green] Path exists")

    # Check if it's a directory
    if not workspace_path.is_dir():
        console.print(f"[red]✗ Path is not a directory[/red]")
        sys.exit(1)
    console.print(f"[green]✓[/green] Path is a directory")

    # Check for XML files
    xml_files = list(workspace_path.rglob("*.xml"))
    if not xml_files:
        console.print(f"[yellow]⚠[/yellow] No XML files found")
    else:
        console.print(f"[green]✓[/green] Found {len(xml_files)} XML file(s)")
        for xml_file in xml_files[:5]:  # Show first 5
            console.print(f"  - {xml_file.name}")
        if len(xml_files) > 5:
            console.print(f"  ... and {len(xml_files) - 5} more")

    console.print()
    console.print("[green]Validation complete![/green]")


def cmd_doctor(args):
    """Check system requirements"""
    import platform
    import subprocess

    console.print("[cyan]System Diagnostics[/cyan]")
    console.print()

    # Python version
    python_version = platform.python_version()
    console.print(f"[green]✓[/green] Python: {python_version}")

    # Platform
    system = platform.system()
    console.print(f"[green]✓[/green] OS: {system}")

    # Java check
    try:
        result = subprocess.run(
            ["java", "-version"],
            capture_output=True,
            text=True,
            check=False,
        )
        if result.returncode == 0:
            # Java version is in stderr
            java_version = result.stderr.split("\n")[0]
            console.print(f"[green]✓[/green] Java: {java_version}")
        else:
            console.print("[yellow]⚠[/yellow] Java not found in PATH")
            console.print("  [dim]Tip: Set auto_download=true in config[/dim]")
    except FileNotFoundError:
        console.print("[yellow]⚠[/yellow] Java not found")
        console.print("  [dim]Tip: Set auto_download=true in config[/dim]")

    # Config file check
    config_path = get_config_path()
    if config_path.exists():
        console.print(f"[green]✓[/green] Config: {config_path}")
    else:
        console.print(f"[yellow]⚠[/yellow] No config file found")
        console.print("  [dim]Run 'eseas run <workspace>' to create one[/dim]")

    # Logs directory
    logs_dir = Path.cwd() / ".eseas" / ".logs"
    if logs_dir.exists():
        log_files = list(logs_dir.glob("*.log"))
        console.print(f"[green]✓[/green] Logs directory: {len(log_files)} log file(s)")
    else:
        console.print("[yellow]⚠[/yellow] No logs directory yet")

    console.print()
    console.print("[green]Diagnostics complete![/green]")


def cmd_logs(args):
    """Show logs"""
    logs_dir = Path.cwd() / ".eseas" / ".logs"

    if not logs_dir.exists():
        console.print("[yellow]No logs directory found[/yellow]")
        console.print("[dim]Logs will be created after first run[/dim]")
        return

    if args.type == "success":
        log_file = logs_dir / "last_good_run.log"
    elif args.type == "error":
        log_file = logs_dir / "failed_runs.log"
    elif args.type == "emergency":
        log_file = logs_dir / "emergency.log"
    else:
        # Show all logs
        console.print(f"[cyan]Logs directory:[/cyan] {logs_dir}")
        console.print()
        for log_file in logs_dir.glob("*.log"):
            size = log_file.stat().st_size
            console.print(f"  {log_file.name} ({size:,} bytes)")
        return

    if not log_file.exists():
        console.print(f"[yellow]Log file not found: {log_file.name}[/yellow]")
        return

    # Show last N lines
    lines = args.lines or 50
    content = log_file.read_text()
    log_lines = content.split("\n")
    last_lines = log_lines[-lines:]

    console.print(f"[cyan]Last {lines} lines from {log_file.name}:[/cyan]")
    console.print()
    for line in last_lines:
        if line.strip():
            console.print(line)


def main():
    """Main CLI entry point"""
    # Get version
    try:
        from eseas import __version__
    except ImportError:
        import importlib.metadata
        __version__ = importlib.metadata.version('eseas')

    parser = argparse.ArgumentParser(
        description="eseas - Seasonal adjustment tool CLI",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  # Show version
  eseas --version

  # First run - creates eseas_config.yaml
  eseas run /path/to/workspace

  # Subsequent runs - uses eseas_config.yaml
  eseas run

  # Use custom config file
  eseas run --config myproject.yaml

  # Override settings
  eseas run --verbose --test

  # Show current configuration
  eseas config show

  # Validate workspace
  eseas validate /path/to/workspace

  # Check system requirements
  eseas doctor

  # View logs
  eseas logs --type error
        """,
    )

    parser.add_argument(
        '--version', '-V',
        action='version',
        version=f'eseas {__version__}'
    )

    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Run command
    run_parser = subparsers.add_parser("run", help="Run seasonal adjustment")
    run_parser.add_argument("workspace", nargs="?", help="Path to Demetra workspace")
    run_parser.add_argument(
        "--config", "-c", help="Path to config file (default: eseas_config.yaml)"
    )
    run_parser.add_argument(
        "--demetra-folder", help="Path to Demetra workspace (alternative to positional)"
    )
    run_parser.add_argument("--output", "-o", help="Output directory")
    run_parser.add_argument("--verbose", "-v", action="store_true", help="Verbose output")
    run_parser.add_argument("--test", "-t", action="store_true", help="Test mode (limit files)")
    run_parser.add_argument("--csvlayout", choices=["vtable", "list"], help="CSV layout format")
    run_parser.set_defaults(func=cmd_run)

    # Config command
    config_parser = subparsers.add_parser("config", help="Manage configuration")
    config_parser.add_argument(
        "action", choices=["show", "set", "init"], help="Config action"
    )
    config_parser.add_argument("--key", help="Configuration key to set")
    config_parser.add_argument("--value", help="Configuration value to set")
    config_parser.add_argument("--workspace", help="Workspace path for init")
    config_parser.add_argument("--config", "-c", help="Config file path")
    config_parser.add_argument("--force", action="store_true", help="Force overwrite for init")
    config_parser.set_defaults(func=cmd_config)

    # Validate command
    validate_parser = subparsers.add_parser("validate", help="Validate workspace")
    validate_parser.add_argument("workspace", help="Path to workspace to validate")
    validate_parser.set_defaults(func=cmd_validate)

    # Doctor command
    doctor_parser = subparsers.add_parser("doctor", help="Check system requirements")
    doctor_parser.set_defaults(func=cmd_doctor)

    # Logs command
    logs_parser = subparsers.add_parser("logs", help="View logs")
    logs_parser.add_argument(
        "--type",
        "-t",
        choices=["success", "error", "emergency"],
        help="Log type to view",
    )
    logs_parser.add_argument(
        "--lines", "-n", type=int, help="Number of lines to show (default: 50)"
    )
    logs_parser.set_defaults(func=cmd_logs)

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        sys.exit(0)

    # Execute command
    args.func(args)


if __name__ == "__main__":
    main()



"""Tests for eseas CLI tool"""

import pytest
import yaml
from pathlib import Path
from unittest.mock import patch, MagicMock, call
import sys

# Import CLI functions
from eseas.cli.main import (
    get_config_path,
    create_default_config,
    load_config,
    save_config,
    cmd_run,
    cmd_config,
    cmd_validate,
    cmd_doctor,
    cmd_logs,
    main,
)


class TestConfigPath:
    """Test configuration path functions"""

    def test_get_config_path(self):
        """Test getting config path"""
        config_path = get_config_path()
        assert config_path.name == "eseas_config.yaml"
        assert config_path.parent == Path.cwd()


class TestDefaultConfig:
    """Test default configuration creation"""

    def test_create_default_config_without_workspace(self):
        """Test creating default config without workspace path"""
        config = create_default_config()
        assert config["demetra_folder"] is None
        assert config["auto_download"] is True
        assert config["local_folder"] == "./eseas_output"
        assert config["csvlayout"] == "vtable"
        assert config["result_file_names"] == ["sa", "s", "cal"]

    def test_create_default_config_with_workspace(self, tmp_path):
        """Test creating default config with workspace path"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()

        config = create_default_config(str(workspace))
        assert config["demetra_folder"] == str(workspace.absolute())
        assert config["auto_download"] is True

    def test_default_config_has_all_required_keys(self):
        """Test that default config has all required keys"""
        config = create_default_config()
        required_keys = [
            "demetra_folder",
            "java_folder",
            "local_folder",
            "auto_download",
            "auto_approve",
            "csvlayout",
            "workspace_mode",
            "result_file_names",
            "verbose",
            "test",
            "replace_general_params",
            "replace_original_files",
            "file_name_explanation",
        ]
        for key in required_keys:
            assert key in config


class TestLoadSaveConfig:
    """Test configuration loading and saving"""

    def test_save_config(self, tmp_path):
        """Test saving configuration to file"""
        config_path = tmp_path / "test_config.yaml"
        config = create_default_config()

        save_config(config, config_path)

        assert config_path.exists()
        loaded = yaml.safe_load(config_path.read_text())
        assert loaded == config

    def test_load_config_existing(self, tmp_path):
        """Test loading existing configuration"""
        config_path = tmp_path / "test_config.yaml"
        test_config = {"key1": "value1", "key2": 123}

        with open(config_path, "w") as f:
            yaml.dump(test_config, f)

        loaded = load_config(config_path)
        assert loaded == test_config

    def test_load_config_nonexistent(self, tmp_path):
        """Test loading non-existent configuration"""
        config_path = tmp_path / "nonexistent.yaml"
        loaded = load_config(config_path)
        assert loaded == {}

    def test_load_config_invalid_yaml(self, tmp_path):
        """Test loading invalid YAML"""
        config_path = tmp_path / "invalid.yaml"
        config_path.write_text("invalid: yaml: content: [")

        with pytest.raises(SystemExit):
            load_config(config_path)

    def test_roundtrip_config(self, tmp_path):
        """Test save and load roundtrip"""
        config_path = tmp_path / "roundtrip.yaml"
        original_config = create_default_config("/test/workspace")

        save_config(original_config, config_path)
        loaded_config = load_config(config_path)

        assert loaded_config == original_config


class TestCmdValidate:
    """Test validate command"""

    def test_validate_existing_workspace(self, tmp_path):
        """Test validating an existing workspace with XML files"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()
        (workspace / "test1.xml").touch()
        (workspace / "test2.xml").touch()

        args = MagicMock()
        args.workspace = str(workspace)

        # Should not raise exception
        cmd_validate(args)

    def test_validate_nonexistent_workspace(self, tmp_path):
        """Test validating non-existent workspace"""
        workspace = tmp_path / "nonexistent"

        args = MagicMock()
        args.workspace = str(workspace)

        with pytest.raises(SystemExit):
            cmd_validate(args)

    def test_validate_file_not_directory(self, tmp_path):
        """Test validating a file instead of directory"""
        not_a_dir = tmp_path / "file.txt"
        not_a_dir.touch()

        args = MagicMock()
        args.workspace = str(not_a_dir)

        with pytest.raises(SystemExit):
            cmd_validate(args)

    def test_validate_empty_workspace(self, tmp_path):
        """Test validating workspace with no XML files"""
        workspace = tmp_path / "empty"
        workspace.mkdir()

        args = MagicMock()
        args.workspace = str(workspace)

        # Should not raise, just warn
        cmd_validate(args)


class TestCmdDoctor:
    """Test doctor command"""

    @patch("subprocess.run")
    @patch("eseas.cli.main.get_config_path")
    def test_doctor_with_java(self, mock_get_config, mock_run, tmp_path):
        """Test doctor when Java is available"""
        # Mock Java check
        mock_result = MagicMock()
        mock_result.returncode = 0
        mock_result.stderr = "java version \"11.0.25\""
        mock_run.return_value = mock_result

        # Mock config path
        config_path = tmp_path / "eseas_config.yaml"
        config_path.touch()
        mock_get_config.return_value = config_path

        args = MagicMock()

        # Should not raise
        cmd_doctor(args)

    @patch("subprocess.run")
    def test_doctor_without_java(self, mock_run):
        """Test doctor when Java is not available"""
        mock_run.side_effect = FileNotFoundError()

        args = MagicMock()

        # Should not raise, just warn
        cmd_doctor(args)


class TestCmdLogs:
    """Test logs command"""

    def test_logs_no_directory(self, tmp_path):
        """Test logs when no logs directory exists"""
        # Change to temp directory
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.type = None
            args.lines = None

            # Should not raise
            cmd_logs(args)
        finally:
            os.chdir(orig_cwd)

    def test_logs_success_type(self, tmp_path):
        """Test viewing success logs"""
        logs_dir = tmp_path / ".eseas" / ".logs"
        logs_dir.mkdir(parents=True)
        log_file = logs_dir / "last_good_run.log"
        log_file.write_text("Test log content\nLine 2\nLine 3")

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.type = "success"
            args.lines = 10

            cmd_logs(args)
        finally:
            os.chdir(orig_cwd)

    def test_logs_error_type(self, tmp_path):
        """Test viewing error logs"""
        logs_dir = tmp_path / ".eseas" / ".logs"
        logs_dir.mkdir(parents=True)
        log_file = logs_dir / "failed_runs.log"
        log_file.write_text("Error log content")

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.type = "error"
            args.lines = 50

            cmd_logs(args)
        finally:
            os.chdir(orig_cwd)

    def test_logs_list_all(self, tmp_path):
        """Test listing all log files"""
        logs_dir = tmp_path / ".eseas" / ".logs"
        logs_dir.mkdir(parents=True)
        (logs_dir / "last_good_run.log").write_text("success")
        (logs_dir / "failed_runs.log").write_text("errors")

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.type = None
            args.lines = None

            cmd_logs(args)
        finally:
            os.chdir(orig_cwd)


class TestCmdConfig:
    """Test config command"""

    def test_config_show_no_file(self, tmp_path):
        """Test showing config when no file exists"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "show"
            args.config = None

            # Should not raise
            cmd_config(args)
        finally:
            os.chdir(orig_cwd)

    def test_config_show_existing(self, tmp_path):
        """Test showing existing config"""
        config_path = tmp_path / "eseas_config.yaml"
        config = create_default_config()
        save_config(config, config_path)

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "show"
            args.config = None

            cmd_config(args)
        finally:
            os.chdir(orig_cwd)

    def test_config_set_value(self, tmp_path):
        """Test setting a config value"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "set"
            args.key = "verbose"
            args.value = "true"
            args.config = None

            cmd_config(args)

            # Verify config was created and value set
            config_path = tmp_path / "eseas_config.yaml"
            assert config_path.exists()
            config = load_config(config_path)
            assert config["verbose"] is True
        finally:
            os.chdir(orig_cwd)

    def test_config_set_list_value(self, tmp_path):
        """Test setting a list value"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "set"
            args.key = "result_file_names"
            args.value = "sa,s,t,cal"
            args.config = None

            cmd_config(args)

            config_path = tmp_path / "eseas_config.yaml"
            config = load_config(config_path)
            assert config["result_file_names"] == ["sa", "s", "t", "cal"]
        finally:
            os.chdir(orig_cwd)

    def test_config_set_boolean_values(self, tmp_path):
        """Test setting boolean values with different representations"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            # Test "true"
            args = MagicMock()
            args.action = "set"
            args.key = "verbose"
            args.value = "true"
            args.config = None
            cmd_config(args)

            config_path = tmp_path / "eseas_config.yaml"
            config = load_config(config_path)
            assert config["verbose"] is True

            # Test "false"
            args.value = "false"
            cmd_config(args)
            config = load_config(config_path)
            assert config["verbose"] is False

            # Test "yes"
            args.value = "yes"
            cmd_config(args)
            config = load_config(config_path)
            assert config["verbose"] is True

            # Test "no"
            args.value = "no"
            cmd_config(args)
            config = load_config(config_path)
            assert config["verbose"] is False
        finally:
            os.chdir(orig_cwd)

    def test_config_init_new(self, tmp_path):
        """Test initializing new config"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "init"
            args.workspace = str(workspace)
            args.config = None
            args.force = False

            cmd_config(args)

            config_path = tmp_path / "eseas_config.yaml"
            assert config_path.exists()
            config = load_config(config_path)
            assert config["demetra_folder"] == str(workspace.absolute())
        finally:
            os.chdir(orig_cwd)

    def test_config_init_existing_no_force(self, tmp_path):
        """Test initializing when config exists without force"""
        config_path = tmp_path / "eseas_config.yaml"
        config_path.write_text("existing: config")

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "init"
            args.workspace = None
            args.config = None
            args.force = False

            # Should not overwrite
            cmd_config(args)

            content = config_path.read_text()
            assert "existing: config" in content
        finally:
            os.chdir(orig_cwd)

    def test_config_init_existing_with_force(self, tmp_path):
        """Test initializing when config exists with force"""
        config_path = tmp_path / "eseas_config.yaml"
        config_path.write_text("existing: config")

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "init"
            args.workspace = None
            args.config = None
            args.force = True

            cmd_config(args)

            # Should be overwritten
            config = load_config(config_path)
            assert "existing" not in config
            assert "demetra_folder" in config
        finally:
            os.chdir(orig_cwd)


class TestCmdRun:
    """Test run command"""

    @patch("eseas.Seasonal")
    @patch("eseas.Options")
    def test_run_first_time_with_workspace(self, mock_options, mock_seasonal, tmp_path):
        """Test first run with workspace path"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.workspace = str(workspace)
            args.demetra_folder = None
            args.config = None
            args.output = None
            args.verbose = None
            args.test = None
            args.csvlayout = None

            cmd_run(args)

            # Verify config was created
            config_path = tmp_path / "eseas_config.yaml"
            assert config_path.exists()

            # Verify Options and Seasonal were called
            mock_options.assert_called_once()
            mock_seasonal.assert_called_once()
            mock_seasonal.return_value.run.assert_called_once()
        finally:
            os.chdir(orig_cwd)

    @patch("eseas.Seasonal")
    @patch("eseas.Options")
    def test_run_with_existing_config(self, mock_options, mock_seasonal, tmp_path):
        """Test run with existing config file"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()

        config_path = tmp_path / "eseas_config.yaml"
        config = create_default_config(str(workspace))
        save_config(config, config_path)

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.workspace = None
            args.demetra_folder = None
            args.config = None
            args.output = None
            args.verbose = None
            args.test = None
            args.csvlayout = None

            cmd_run(args)

            mock_options.assert_called_once()
            mock_seasonal.assert_called_once()
        finally:
            os.chdir(orig_cwd)

    def test_run_no_workspace_no_config(self, tmp_path):
        """Test run without workspace and without config"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.workspace = None
            args.demetra_folder = None
            args.config = None
            args.output = None
            args.verbose = None
            args.test = None
            args.csvlayout = None

            with pytest.raises(SystemExit):
                cmd_run(args)
        finally:
            os.chdir(orig_cwd)

    @patch("eseas.Seasonal")
    @patch("eseas.Options")
    def test_run_with_overrides(self, mock_options, mock_seasonal, tmp_path):
        """Test run with command-line overrides"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()

        config_path = tmp_path / "eseas_config.yaml"
        config = create_default_config(str(workspace))
        save_config(config, config_path)

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.workspace = None
            args.demetra_folder = None
            args.config = None
            args.output = "/custom/output"
            args.verbose = True
            args.test = True
            args.csvlayout = "list"

            cmd_run(args)

            # Verify Options was called with overrides
            call_kwargs = mock_options.call_args[1]
            assert call_kwargs["local_folder"] == "/custom/output"
            assert call_kwargs["verbose"] is True
            assert call_kwargs["test"] is True
            assert call_kwargs["csvlayout"] == "list"
        finally:
            os.chdir(orig_cwd)

    @patch("eseas.Seasonal")
    @patch("eseas.Options")
    def test_run_custom_config_file(self, mock_options, mock_seasonal, tmp_path):
        """Test run with custom config file"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()

        custom_config = tmp_path / "custom.yaml"
        config = create_default_config(str(workspace))
        save_config(config, custom_config)

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.workspace = None
            args.demetra_folder = None
            args.config = str(custom_config)
            args.output = None
            args.verbose = None
            args.test = None
            args.csvlayout = None

            cmd_run(args)

            mock_options.assert_called_once()
            mock_seasonal.assert_called_once()
        finally:
            os.chdir(orig_cwd)

    def test_run_nonexistent_custom_config(self, tmp_path):
        """Test run with non-existent custom config file"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.workspace = None
            args.demetra_folder = None
            args.config = "nonexistent.yaml"
            args.output = None
            args.verbose = None
            args.test = None
            args.csvlayout = None

            with pytest.raises(SystemExit):
                cmd_run(args)
        finally:
            os.chdir(orig_cwd)


class TestMainCLI:
    """Test main CLI entry point"""

    def test_main_no_args(self):
        """Test main with no arguments"""
        with patch("sys.argv", ["eseas"]):
            with pytest.raises(SystemExit) as exc_info:
                main()
            assert exc_info.value.code == 0

    @patch("eseas.cli.main.cmd_doctor")
    def test_main_doctor_command(self, mock_cmd):
        """Test main with doctor command"""
        with patch("sys.argv", ["eseas", "doctor"]):
            main()
            mock_cmd.assert_called_once()

    @patch("eseas.cli.main.cmd_validate")
    def test_main_validate_command(self, mock_cmd):
        """Test main with validate command"""
        with patch("sys.argv", ["eseas", "validate", "/path/to/workspace"]):
            main()
            mock_cmd.assert_called_once()

    @patch("eseas.cli.main.cmd_config")
    def test_main_config_show_command(self, mock_cmd):
        """Test main with config show command"""
        with patch("sys.argv", ["eseas", "config", "show"]):
            main()
            mock_cmd.assert_called_once()

    def test_main_version_flag(self):
        """Test main with --version flag"""
        with patch("sys.argv", ["eseas", "--version"]):
            with pytest.raises(SystemExit) as exc_info:
                main()
            assert exc_info.value.code == 0

    def test_main_version_short_flag(self):
        """Test main with -V flag"""
        with patch("sys.argv", ["eseas", "-V"]):
            with pytest.raises(SystemExit) as exc_info:
                main()
            assert exc_info.value.code == 0


class TestConfigEdgeCases:
    """Test edge cases in configuration handling"""

    def test_config_with_special_characters(self, tmp_path):
        """Test config with special characters in paths"""
        workspace = tmp_path / "workspace with spaces"
        workspace.mkdir()

        config = create_default_config(str(workspace))
        config_path = tmp_path / "test.yaml"
        save_config(config, config_path)

        loaded = load_config(config_path)
        assert "with spaces" in loaded["demetra_folder"]

    def test_config_with_unicode(self, tmp_path):
        """Test config with unicode characters"""
        config = create_default_config()
        config["demetra_folder"] = "/path/with/ğüşıöç/chars"

        config_path = tmp_path / "unicode.yaml"
        save_config(config, config_path)

        loaded = load_config(config_path)
        assert "ğüşıöç" in loaded["demetra_folder"]

    def test_config_set_without_key_value(self, tmp_path):
        """Test config set without key or value"""
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args = MagicMock()
            args.action = "set"
            args.key = None
            args.value = None
            args.config = None

            with pytest.raises(SystemExit):
                cmd_config(args)
        finally:
            os.chdir(orig_cwd)


class TestIntegration:
    """Integration tests for CLI workflows"""

    @patch("eseas.Seasonal")
    @patch("eseas.Options")
    def test_full_workflow_first_run_then_rerun(self, mock_options, mock_seasonal, tmp_path):
        """Test complete workflow: first run, then subsequent run"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()
        (workspace / "test.xml").touch()

        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            # First run
            args1 = MagicMock()
            args1.workspace = str(workspace)
            args1.demetra_folder = None
            args1.config = None
            args1.output = None
            args1.verbose = None
            args1.test = None
            args1.csvlayout = None

            cmd_run(args1)

            # Verify config was created
            config_path = tmp_path / "eseas_config.yaml"
            assert config_path.exists()

            # Second run (no workspace arg)
            args2 = MagicMock()
            args2.workspace = None
            args2.demetra_folder = None
            args2.config = None
            args2.output = None
            args2.verbose = None
            args2.test = None
            args2.csvlayout = None

            cmd_run(args2)

            # Should have been called twice
            assert mock_seasonal.call_count == 2
        finally:
            os.chdir(orig_cwd)

    def test_validate_then_run_workflow(self, tmp_path):
        """Test workflow: validate workspace, then run"""
        workspace = tmp_path / "workspace"
        workspace.mkdir()
        (workspace / "test.xml").touch()

        # Validate
        args_validate = MagicMock()
        args_validate.workspace = str(workspace)
        cmd_validate(args_validate)  # Should not raise

        # Then config init
        import os
        orig_cwd = os.getcwd()
        try:
            os.chdir(tmp_path)

            args_init = MagicMock()
            args_init.action = "init"
            args_init.workspace = str(workspace)
            args_init.config = None
            args_init.force = False

            cmd_config(args_init)

            config_path = tmp_path / "eseas_config.yaml"
            assert config_path.exists()
        finally:
            os.chdir(orig_cwd)



from pathlib import Path
import os
import pandas as pd
import pytest

from eseas.core.collect import collect_parts_of_results, make_float
from eseas.core.seasonal_options import SeasonalOptions as Options

def test_make_float():
    df = pd.DataFrame(
        {
            "Unnamed: 0": ["0", "1"],
            "a": ["1,12", "12,00"],
            "b": ["15,0", "20,0"],
        }
    )
    df_float = make_float(df)
    assert df_float["a"].iloc[0] == 1.12
    assert df_float["a"].iloc[1] == 12.0
    assert df_float["b"].iloc[0] == 15.0
    assert df_float["b"].iloc[1] == 20.0

def test_collect_parts_of_results(tmp_path, monkeypatch):
    # Mock get_cruncher to return a dummy cruncher with local_work_space
    class DummyCruncher:
        def __init__(self, ws):
            self.local_work_space = str(ws)

    # Use a separate mock workspace for our inputs
    workspace_dir = tmp_path / "workspace"
    workspace_dir.mkdir()
    
    import eseas.core.collect as collect_mod
    monkeypatch.setattr(collect_mod, "get_cruncher", lambda: DummyCruncher(workspace_dir))
    
    # We will simulate collecting two folders: f1, f2
    for parent in ["f1", "f2"]:
        # create mock files inside our mock workspace (where get_cruncher().local_work_space points)
        dir_path = workspace_dir / "test_output" / parent / "SAProcessing-1"
        dir_path.mkdir(parents=True, exist_ok=True)
        
        # Create Dummy CSVs for default parts
        for part in ["sa", "s", "cal"]:
            df = pd.DataFrame({"Unnamed: 0": ["0"], f"series_{part}": ["1,0"]})
            df.to_csv(dir_path / f"series_{part}.csv", index=False, sep=";")
            
    # Mock Demetra folder search if needed, but we can pass xml_folders directly
    options = Options(
        demetra_folder=str(tmp_path),
        local_folder=str(tmp_path / "final_output"),
        result_file_names=("sa", "s", "cal")
    )
    
    # Run the function
    collect_parts_of_results(options, xml_folders=["f1", "f2"])

    # Check that final combined excels were written to options.local_folder
    final_output = tmp_path / "final_output" / "test_output"
    
    f1_excel = final_output / "f1" / "combined.xlsx"
    assert f1_excel.exists()
    assert set(pd.ExcelFile(f1_excel).sheet_names) == {"sa", "s", "cal"}
    
    f2_excel = final_output / "f2" / "combined.xlsx"
    assert f2_excel.exists()
    assert set(pd.ExcelFile(f2_excel).sheet_names) == {"sa", "s", "cal"}

def test_collect_parts_out_folder(tmp_path, monkeypatch):
    class DummyCruncher:
        def __init__(self, ws):
            self.local_work_space = str(ws)

    workspace_dir = tmp_path / "workspace"
    workspace_dir.mkdir()
    
    import eseas.core.collect as collect_mod
    monkeypatch.setattr(collect_mod, "get_cruncher", lambda: DummyCruncher(workspace_dir))
    
    dir_path = workspace_dir / "test_output" / "f3" / "SAProcessing-1"
    dir_path.mkdir(parents=True, exist_ok=True)
    
    for part in ["sa"]:
        df = pd.DataFrame({"Unnamed: 0": ["0"], f"series_{part}": ["1,0"]})
        df.to_csv(dir_path / f"series_{part}.csv", index=False, sep=";")
        
    options = Options(
        demetra_folder=str(tmp_path),
        local_folder=str(tmp_path / "final_output"),
        result_file_names=("sa",)
    )
    
    custom_out = tmp_path / "custom_out"
    
    collect_parts_of_results(options, xml_folders=["f3"], out_folder=str(custom_out), out_file_name="special")
    
    f3_excel = custom_out / "f3" / "special.xlsx"
    assert f3_excel.exists()
    assert set(pd.ExcelFile(f3_excel).sheet_names) == {"sa"}




import urllib.request
import zipfile
import time
from pathlib import Path

CRUNCHER_VERSION = "2.2.5"
CRUNCHER_URL = f"https://github.com/jdemetra/jwsacruncher/releases/download/v{CRUNCHER_VERSION}/jwsacruncher-{CRUNCHER_VERSION}-bin.zip"
MAX_RETRIES = 3
RETRY_DELAY = 2  # seconds

def download_jwsacruncher(target_dir: str | Path) -> Path:
    """
    Download and extract jwsacruncher to the specified directory.
    Returns the path to the extracted 'bin' folder.
    """
    target_dir = Path(target_dir)
    # The actual bin path after extraction will be target_dir/jwsacruncher-{CRUNCHER_VERSION}/bin
    expected_bin_path = target_dir / f"jwsacruncher-{CRUNCHER_VERSION}" / "bin"
    
    if expected_bin_path.exists() and expected_bin_path.is_dir():
        print(f"jwsacruncher already available at {expected_bin_path}")
        return expected_bin_path

    print(f"\nDownloading jwsacruncher {CRUNCHER_VERSION} to {target_dir}...")
    target_dir.mkdir(parents=True, exist_ok=True)
    zip_file = target_dir / f"jwsacruncher-{CRUNCHER_VERSION}-bin.zip"

    for attempt in range(MAX_RETRIES):
        try:
            print(f"Download attempt {attempt + 1}/{MAX_RETRIES}...")
            
            # Custom request with User-Agent
            req = urllib.request.Request(
                CRUNCHER_URL,
                headers={'User-Agent': 'Mozilla/5.0'}
            )
            
            with urllib.request.urlopen(req, timeout=60) as response:
                with open(zip_file, 'wb') as out_file:
                    out_file.write(response.read())
            
            print(f"Downloaded to {zip_file}")
            print(f"Extracting to {target_dir}...")
            
            with zipfile.ZipFile(zip_file, "r") as zip_ref:
                zip_ref.extractall(target_dir)

            zip_file.unlink()  # Remove zip file after extraction
            
            # Make the unix script executable
            import os
            import stat
            unix_cruncher = expected_bin_path / "jwsacruncher"
            if unix_cruncher.exists():
                st = os.stat(unix_cruncher)
                os.chmod(unix_cruncher, st.st_mode | stat.S_IEXEC)
                
            print(f"jwsacruncher extracted successfully. Bin path: {expected_bin_path}")
            return expected_bin_path

        except Exception as e:
            print(f"Download attempt {attempt + 1} failed: {e}")
            if zip_file.exists():
                try:
                    zip_file.unlink()
                except Exception:
                    pass
            
            if attempt < MAX_RETRIES - 1:
                print(f"Waiting {RETRY_DELAY} seconds before retry...")
                time.sleep(RETRY_DELAY)
            else:
                raise RuntimeError(f"Failed to download jwsacruncher after {MAX_RETRIES} attempts.")



import pandas as pd
from io import StringIO
from urllib.parse import quote
from pathlib import Path
import re
import zipfile
import xml.etree.ElementTree as ET
from eseas.data_for_testing.csv_content import air_passenger


def _fix_xlsx_shared_strings(xlsx_path: Path) -> None:
    """
    openpyxl 3.1+ writes strings as inlineStr cells, omitting sharedStrings.xml.
    JDemetra+'s XCLPRVDR parser requires sharedStrings.xml to be present.
    This function converts inline strings to a proper shared strings table in-place.
    """
    import io

    with zipfile.ZipFile(xlsx_path, 'r') as zin:
        names = zin.namelist()
        files = {name: zin.read(name) for name in names}

    sheet_xml = files.get('xl/worksheets/sheet1.xml', b'')
    tree = ET.fromstring(sheet_xml)
    ns = 'http://schemas.openxmlformats.org/spreadsheetml/2006/main'

    strings: list[str] = []
    index_map: dict[str, int] = {}

    def intern_string(value: str) -> int:
        if value not in index_map:
            index_map[value] = len(strings)
            strings.append(value)
        return index_map[value]

    # Convert inlineStr cells to shared-string references
    for c in tree.iter(f'{{{ns}}}c'):
        if c.attrib.get('t') == 'inlineStr':
            is_elem = c.find(f'{{{ns}}}is')
            t_elem = is_elem.find(f'{{{ns}}}t') if is_elem is not None else None
            value = t_elem.text if t_elem is not None else ''
            idx = intern_string(value or '')
            c.attrib['t'] = 's'
            for child in list(c):
                c.remove(child)
            v_elem = ET.SubElement(c, f'{{{ns}}}v')
            v_elem.text = str(idx)

    if not strings:
        return  # nothing to do

    # Rebuild the sheet XML bytes
    files['xl/worksheets/sheet1.xml'] = ET.tostring(tree, encoding='unicode', xml_declaration=False).encode('utf-8')

    # Build sharedStrings.xml
    ss_root = ET.Element('sst')
    ss_root.attrib['xmlns'] = 'http://schemas.openxmlformats.org/spreadsheetml/2006/main'
    ss_root.attrib['count'] = str(len(strings))
    ss_root.attrib['uniqueCount'] = str(len(strings))
    for s in strings:
        si = ET.SubElement(ss_root, 'si')
        t = ET.SubElement(si, 't')
        t.text = s
    ss_xml = b'<?xml version="1.0" encoding="UTF-8" standalone="yes"?>\n' + ET.tostring(ss_root, encoding='unicode').encode('utf-8')
    files['xl/sharedStrings.xml'] = ss_xml

    # Update [Content_Types].xml to register sharedStrings
    ct_xml = files['[Content_Types].xml'].decode('utf-8')
    if 'sharedStrings' not in ct_xml:
        ct_xml = ct_xml.replace(
            '</Types>',
            '<Override PartName="/xl/sharedStrings.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml"/></Types>'
        )
        files['[Content_Types].xml'] = ct_xml.encode('utf-8')

    # Update xl/_rels/workbook.xml.rels:
    #  1. Convert absolute targets like "/xl/foo" → "foo" (relative to xl/ base)
    #     JDemetra's xlsx reader prepends "xl/" to every target, so absolute paths
    #     like "/xl/worksheets/sheet1.xml" would become "xl//xl/worksheets/sheet1.xml".
    #  2. Add the sharedStrings relationship.
    rels_xml = files['xl/_rels/workbook.xml.rels'].decode('utf-8')
    rels_xml = re.sub(r'Target="/xl/', 'Target="', rels_xml)
    if 'sharedStrings' not in rels_xml:
        rels_xml = rels_xml.replace(
            '</Relationships>',
            '<Relationship Id="rIdSS" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/sharedStrings" Target="sharedStrings.xml"/></Relationships>'
        )
    files['xl/_rels/workbook.xml.rels'] = rels_xml.encode('utf-8')

    # Write back
    buf = io.BytesIO()
    with zipfile.ZipFile(buf, 'w', zipfile.ZIP_DEFLATED) as zout:
        for name, data in files.items():
            zout.writestr(name, data)
    xlsx_path.write_bytes(buf.getvalue())


def apply_test_patch(base_dir: Path):
    if not base_dir.exists():
        return
    
    # JDemetra requires explicit XCLPRVDR URL encoded components. Using standard
    # quote doesn't encode forward slashes. Force encode slashes for the test XML file
    df = pd.read_csv(StringIO(air_passenger), sep=';')
    excel_path = base_dir / 'air4.xlsx'
    df.to_excel(excel_path, sheet_name="Sheet1", index=False)
    _fix_xlsx_shared_strings(excel_path)
    
    abs_path = str(excel_path.absolute())
    mac_url_part = quote(abs_path, safe="") # strict encoding!
    excel_url = f'file={mac_url_part}'
    
    for xml_path in base_dir.rglob("*.xml"):
        content = xml_path.read_text()
        if "file=" in content and "air4.xlsx" in content:
            new_content = re.sub(r'file=[a-zA-Z0-9%_\-\./]+xlsx', excel_url, content)
            xml_path.write_text(new_content)



import json
import logging
from pathlib import Path
from datetime import datetime
import traceback
import sys
import platform
import os

def setup_eseas_logger(name="eseas_error_logger", filename="failed_runs.log", level=logging.ERROR):
    """Setup logger with guaranteed directory creation"""
    cwd = Path.cwd()
    eseas_dir = cwd / ".eseas"
    logs_dir = eseas_dir / ".logs"

    try:
        logs_dir.mkdir(parents=True, exist_ok=True)
    except Exception as e:
        # Fallback to temp directory if unable to create in cwd
        import tempfile
        eseas_dir = Path(tempfile.gettempdir()) / ".eseas"
        logs_dir = eseas_dir / ".logs"
        logs_dir.mkdir(parents=True, exist_ok=True)
        print(f"Warning: Using temporary directory for logs: {logs_dir}")
        print(f"Original error: {e}")
    
    example_script = eseas_dir / "quick_start_example.py"
    if not example_script.exists():
        example_content = r"""from eseas import Options, Seasonal

def main():
    options = Options(
        demetra_source_folder=r"C:\Data\your_demetra_xmls",
        local_folder=r"C:\Data\eseas_outputs", 
        auto_download=True,
        auto_approve=False
    )
    
    seas = Seasonal(options)
    seas.run()

if __name__ == "__main__":
    main()
"""
        try:
            example_script.write_text(example_content, encoding='utf-8')
        except Exception:
            pass

    readme_file = eseas_dir / "README.txt"
    if not readme_file.exists():
        readme_content = """This .eseas directory is automatically generated by the eseas package.
It contains diagnostic logs in the .logs/ folder to assist with troubleshooting.
"""
        try:
            readme_file.write_text(readme_content, encoding='utf-8')
        except Exception:
            pass
            
    logger = logging.getLogger(name)
    if not logger.handlers:
        logger.setLevel(level)
        
        log_file = logs_dir / filename
        handler = logging.FileHandler(log_file, encoding='utf-8')
        handler.setLevel(level)
        
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        
        logger.addHandler(handler)
        
    return logger

def get_system_info():
    """Get comprehensive system information"""
    info = {}
    try:
        info["platform"] = platform.platform()
        info["python_version"] = sys.version
        info["python_executable"] = sys.executable
        info["cwd"] = str(Path.cwd())
        info["user"] = os.environ.get("USER") or os.environ.get("USERNAME") or "unknown"
        info["timestamp"] = datetime.now().isoformat()

        # Check Java availability
        try:
            import subprocess
            result = subprocess.run(["java", "-version"], capture_output=True, text=True, timeout=5)
            info["java_available"] = True
            info["java_version"] = result.stderr.split('\n')[0] if result.stderr else "unknown"
        except Exception:
            info["java_available"] = False
            info["java_version"] = "not found"

    except Exception as e:
        info["error"] = f"Could not gather system info: {e}"

    return info

def get_paths_info():
    """Get workspace and execution paths"""
    paths = {}
    try:
        from .cruncher_classes import get_cruncher
        from .create_bat_command import get_demetra_type

        cruncher = get_cruncher()
        ws = cruncher.local_work_space
        paths["local_workspace"] = str(ws)
        paths["demetra_folder"] = str(cruncher.demetra_folder)
        paths["cruncher_folder"] = str(cruncher.crunch_folder)
        paths["general_params"] = str(Path(ws) / "general.params")

        # Check if paths exist
        paths["workspace_exists"] = Path(ws).exists()
        paths["demetra_folder_exists"] = Path(cruncher.demetra_folder).exists()
        paths["cruncher_folder_exists"] = Path(cruncher.crunch_folder).exists()

        try:
            paths["execution_script"] = str(get_demetra_type().demetra_command_file_name())
        except Exception:
            paths["execution_script"] = "Could not resolve"

        # check script content
        try:
            exec_path = Path(paths["execution_script"])
            if exec_path.exists():
                paths["script_content"] = exec_path.read_text(encoding="utf-8")
                paths["script_exists"] = True
            else:
                paths["script_exists"] = False
        except Exception as e:
            paths["script_read_error"] = str(e)

    except Exception as e:
        paths["error"] = f"Could not resolve workspace paths: {e}"
        paths["traceback"] = traceback.format_exc()

    return paths

def get_processing_stats():
    """Get statistics about what was processed"""
    stats = {}
    try:
        from .cruncher_classes import get_cruncher
        cruncher = get_cruncher()
        ws = Path(cruncher.local_work_space)

        # Count XML files in demetra folder
        if Path(cruncher.demetra_folder).exists():
            xml_files = list(Path(cruncher.demetra_folder).rglob("*.xml"))
            stats["xml_files_found"] = len(xml_files)
            stats["xml_file_list"] = [str(f.name) for f in xml_files[:20]]  # Limit to first 20

        # Count output files
        if ws.exists():
            test_output = ws / "test_output"
            if test_output.exists():
                csv_files = list(test_output.rglob("*.csv"))
                xlsx_files = list(test_output.rglob("*.xlsx"))
                stats["csv_outputs_generated"] = len(csv_files)
                stats["xlsx_outputs_generated"] = len(xlsx_files)

    except Exception as e:
        stats["error"] = f"Could not gather processing stats: {e}"

    return stats

def log_eseas_run(status: str, error_msg: str=None, options=None, execution_time: float=None):
    """
    Comprehensive logging for eseas runs

    Args:
        status: 'success' or 'error'
        error_msg: Error message with traceback
        options: SeasonalOptions instance
        execution_time: Time taken in seconds
    """
    try:
        # Get version
        try:
            from .. import __version__
            eseas_version = __version__
        except ImportError:
            try:
                import importlib.metadata
                eseas_version = importlib.metadata.version('eseas')
            except Exception:
                eseas_version = "Unknown"

        # Setup logger
        if status == "success":
            logger = setup_eseas_logger(name="eseas_success_logger", filename="last_good_run.log", level=logging.INFO)
        else:
            logger = setup_eseas_logger()

        # Get options
        options_dict = {}
        if options:
            try:
                if hasattr(options, "__dict__"):
                    for k, v in options.__dict__.items():
                        options_dict[k] = str(v)
                else:
                    options_dict = {"repr": repr(options)}
            except Exception as e:
                options_dict = {"repr": "Could not parse options.", "error": str(e)}

        # Gather comprehensive information
        system_info = get_system_info()
        paths_info = get_paths_info()
        stats = get_processing_stats()

        # Build log data
        log_data = {
            "status": status,
            "eseas_version": eseas_version,
            "timestamp": datetime.now().isoformat(),
            "execution_time_seconds": execution_time if execution_time else "not_measured",
            "system": system_info,
            "options": options_dict,
            "paths": paths_info,
            "processing_stats": stats
        }

        if error_msg:
            log_data["error"] = {
                "message": str(error_msg),
                "type": "runtime_error"
            }

        # Log with appropriate level
        if status == "success":
            logger.info(json.dumps(log_data, indent=2))
        else:
            logger.error(json.dumps(log_data, indent=2))

        return True

    except Exception as e:
        # Emergency fallback - write to a simple text file if JSON logging fails
        try:
            emergency_log = Path.cwd() / ".eseas" / ".logs" / "emergency.log"
            emergency_log.parent.mkdir(parents=True, exist_ok=True)
            with open(emergency_log, "a", encoding="utf-8") as f:
                f.write(f"\n{'='*80}\n")
                f.write(f"EMERGENCY LOG ENTRY - {datetime.now().isoformat()}\n")
                f.write(f"Logging system failed with error: {e}\n")
                f.write(f"Original status: {status}\n")
                f.write(f"Original error: {error_msg}\n")
                f.write(f"Traceback:\n{traceback.format_exc()}\n")
                f.write(f"{'='*80}\n")
        except Exception:
            # Absolute last resort - print to stderr
            print(f"CRITICAL: All logging mechanisms failed!", file=sys.stderr)
            print(f"Error: {e}", file=sys.stderr)
            if error_msg:
                print(f"Original error: {error_msg}", file=sys.stderr)

        return False

def log_eseas_error(error_msg: str, options=None, execution_time: float=None):
    """Log an error with comprehensive diagnostics"""
    log_eseas_run("error", error_msg=error_msg, options=options, execution_time=execution_time)

def log_eseas_success(options=None, execution_time: float=None):
    """Log a successful run with statistics"""
    log_eseas_run("success", options=options, execution_time=execution_time)

def ensure_log_on_exception(func):
    """
    Decorator to ensure logging happens even if an exception occurs.
    This provides a central failure point to always create log files.

    Usage:
        @ensure_log_on_exception
        def some_function(self):
            # your code
    """
    def wrapper(*args, **kwargs):
        import time
        start_time = time.time()
        options = None

        # Try to extract options from args
        try:
            if args and hasattr(args[0], 'options'):
                options = args[0].options
        except Exception:
            pass

        try:
            result = func(*args, **kwargs)
            # Don't log here if the function logs internally
            return result
        except Exception as e:
            execution_time = time.time() - start_time
            error_msg = f"{str(e)}\n{traceback.format_exc()}"
            log_eseas_error(error_msg, options, execution_time)
            raise

    return wrapper

def install_global_exception_handler():
    """
    Install a global exception handler to catch any uncaught exceptions
    and ensure they are logged before the program exits.
    """
    original_excepthook = sys.excepthook

    def eseas_excepthook(exc_type, exc_value, exc_traceback):
        """Custom exception hook that logs uncaught exceptions"""
        # Format the exception
        error_lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
        error_msg = ''.join(error_lines)

        # Try to log it
        try:
            log_eseas_error(error_msg, options=None)
        except Exception:
            # If logging fails, at least try emergency log
            try:
                emergency_log = Path.cwd() / ".eseas" / ".logs" / "emergency.log"
                emergency_log.parent.mkdir(parents=True, exist_ok=True)
                with open(emergency_log, "a", encoding="utf-8") as f:
                    f.write(f"\n{'='*80}\n")
                    f.write(f"UNCAUGHT EXCEPTION - {datetime.now().isoformat()}\n")
                    f.write(error_msg)
                    f.write(f"{'='*80}\n")
            except Exception:
                pass

        # Call the original exception hook
        original_excepthook(exc_type, exc_value, exc_traceback)

    sys.excepthook = eseas_excepthook




"""Tests for eseas.core.json_ops module"""

import json
import pytest
from pathlib import Path
from unittest.mock import patch, MagicMock
from eseas.core.json_ops import (
    dict_apply,
    remove_json_bad_chars,
    replace_recursive_json,
    create_json_file,
)


class TestDictApply:
    """Test dict_apply function"""

    def test_dict_apply_with_dict(self):
        """Test applying function to dictionary values"""
        input_dict = {"key1": "value1", "key2": "value2"}
        result = dict_apply(input_dict, str.upper)
        assert result == {"key1": "VALUE1", "key2": "VALUE2"}

    def test_dict_apply_with_list(self):
        """Test applying function to list elements"""
        input_list = ["hello", "world"]
        result = dict_apply(input_list, str.upper)
        assert result == ["HELLO", "WORLD"]

    def test_dict_apply_with_tuple(self):
        """Test applying function to tuple elements"""
        input_tuple = ("hello", "world")
        result = dict_apply(input_tuple, str.upper)
        assert result == ["HELLO", "WORLD"]  # Returns list

    def test_dict_apply_with_nested_values(self):
        """Test that non-string values are ignored"""
        input_dict = {"str_val": "text", "int_val": 123, "bool_val": True}
        result = dict_apply(input_dict, str.upper)
        assert result == {"str_val": "TEXT"}
        assert "int_val" not in result
        assert "bool_val" not in result


class TestReplaceRecursiveJson:
    """Test replace_recursive_json function"""

    def test_simple_replacement(self):
        """Test simple character replacement"""
        text = "Hello$World$Test"
        result = replace_recursive_json(text, "$", "USD")
        assert result == "HelloUSDWorldUSDTest"

    def test_no_replacement_needed(self):
        """Test when character is not present"""
        text = "HelloWorld"
        result = replace_recursive_json(text, "$", "USD")
        assert result == "HelloWorld"

    def test_multiple_occurrences(self):
        """Test multiple occurrences of same character"""
        text = "$$$"
        result = replace_recursive_json(text, "$", "USD")
        # Splitting "$$$" by "$" gives ["", "", "", ""], joining with "USD" gives "USDUSDUSD"
        assert result == "USDUSDUSD"

    def test_replacement_with_special_chars(self):
        """Test replacement with special characters"""
        text = "Test?Question?Mark"
        result = replace_recursive_json(text, "?", "QUESTIONMARK")
        assert result == "TestQUESTIONMARKQuestionQUESTIONMARKMark"


class TestRemoveJsonBadChars:
    """Test remove_json_bad_chars function"""

    def test_remove_bad_chars_from_string(self):
        """Test removing bad characters from string"""
        text = "Price: $100 (50% off)"
        result = remove_json_bad_chars(text)
        assert "$" not in result
        assert "%" not in result
        assert "USD" in result
        assert "PERCENT" in result

    def test_remove_bad_chars_from_dict(self):
        """Test removing bad characters from dictionary values"""
        input_dict = {
            "price": "$100",
            "discount": "50%",
            "name": "Product",
        }
        result = remove_json_bad_chars(input_dict)
        assert "$" not in result["price"]
        assert "USD" in result["price"]
        assert "%" not in result["discount"]
        assert "PERCENT" in result["discount"]
        assert result["name"] == "Product"  # No change

    def test_remove_bad_chars_from_list(self):
        """Test removing bad characters from list"""
        input_list = ["$100", "50%", "&symbol"]
        result = remove_json_bad_chars(input_list)
        assert "$" not in result[0]
        assert "USD" in result[0]
        assert "%" not in result[1]
        assert "AMPERSTAND" in result[2]

    def test_remove_bad_chars_from_tuple(self):
        """Test removing bad characters from tuple"""
        input_tuple = ("$100", "50%")
        result = remove_json_bad_chars(input_tuple)
        assert isinstance(result, list)  # Converts to list
        assert "$" not in result[0]
        assert "USD" in result[0]

    def test_all_bad_chars_replaced(self):
        """Test that all defined bad characters are replaced"""
        text = "$100 50% A&B Question? ÄŸ Ä"
        result = remove_json_bad_chars(text)
        assert "$" not in result
        assert "%" not in result
        assert "&" not in result
        assert "?" not in result
        assert "USD" in result
        assert "PERCENT" in result
        assert "AMPERSTAND" in result
        assert "QUESTIONMARK" in result

    def test_nested_dict_processing(self):
        """Test processing nested dictionary"""
        input_dict = {
            "level1": "$100",
            "nested": {
                "level2": "50%",
            },
        }
        result = remove_json_bad_chars(input_dict)
        # Only processes top-level string values
        assert "USD" in result["level1"]
        # Nested dict is not processed (by design of dict_apply)
        assert "nested" not in result


class TestCreateJsonFile:
    """Test create_json_file function"""

    @patch("evdspy.EVDSlocal.common.files.Write")
    def test_create_json_file_default_name(self, mock_write, tmp_path):
        """Test creating JSON file with default name"""
        test_dict = {"key1": "value1", "key2": "value2"}
        mock_write.return_value = (True, "Success")

        with patch("eseas.core.json_ops.Path", return_value=tmp_path):
            create_json_file(test_dict)

        # Should be called twice (two Write calls in the function)
        assert mock_write.call_count == 2

        # Check the format of the written content
        call_args = mock_write.call_args_list[0][0]
        content = call_args[1]
        assert content.startswith("file_and_cols='")
        assert content.endswith("'")
        assert "key1" in content
        assert "value1" in content

    @patch("evdspy.EVDSlocal.common.files.Write")
    def test_create_json_file_custom_name(self, mock_write, tmp_path):
        """Test creating JSON file with custom name"""
        test_dict = {"test": "data"}
        custom_name = "custom.js"
        mock_write.return_value = (True, "Success")

        with patch("eseas.core.json_ops.Path", return_value=tmp_path):
            create_json_file(test_dict, file_name=custom_name)

        assert mock_write.call_count == 2

    @patch("evdspy.EVDSlocal.common.files.Write")
    def test_create_json_file_with_unicode(self, mock_write, tmp_path):
        """Test creating JSON file with unicode characters"""
        test_dict = {"name": "Test", "description": "Unicode: ğüşıöç"}
        mock_write.return_value = (True, "Success")

        with patch("eseas.core.json_ops.Path", return_value=tmp_path):
            create_json_file(test_dict)

        # Verify ensure_ascii=False is used (unicode preserved)
        call_args = mock_write.call_args_list[0][0]
        content = call_args[1]
        # Unicode characters should be preserved
        assert "ğüşıöç" in content or "Unicode" in content

    @patch("evdspy.EVDSlocal.common.files.Write")
    def test_create_json_file_empty_dict(self, mock_write, tmp_path):
        """Test creating JSON file with empty dictionary"""
        test_dict = {}
        mock_write.return_value = (True, "Success")

        with patch("eseas.core.json_ops.Path", return_value=tmp_path):
            create_json_file(test_dict)

        call_args = mock_write.call_args_list[0][0]
        content = call_args[1]
        assert content == "file_and_cols='{}'"


class TestIntegration:
    """Integration tests combining multiple functions"""

    def test_clean_and_create_json_file(self, tmp_path):
        """Test cleaning bad chars and creating JSON file"""
        dirty_dict = {
            "price": "$100",
            "discount": "50%",
            "question": "What?",
        }

        # Clean the dictionary
        clean_dict = remove_json_bad_chars(dirty_dict)

        # Verify cleaning worked
        assert "$" not in clean_dict.get("price", "")
        assert "%" not in clean_dict.get("discount", "")

        # Verify it's valid JSON-serializable
        json_str = json.dumps(clean_dict)
        assert "USD" in json_str
        assert "PERCENT" in json_str

    def test_dict_apply_with_cleaning_function(self):
        """Test using dict_apply with cleaning function"""
        input_dict = {"val1": "$100", "val2": "50%"}

        def clean_value(text):
            return text.replace("$", "USD").replace("%", "PERCENT")

        result = dict_apply(input_dict, clean_value)
        assert result["val1"] == "USD100"
        assert result["val2"] == "50PERCENT"



"""Tests for eseas.core.refresh_excel module"""

import pytest
import sys
from pathlib import Path
from unittest.mock import patch, MagicMock, call
from eseas.core.refresh_excel import (
    ask_permission,
    excel_running,
    File,
)

# Conditionally import Windows-specific functions
if sys.platform == "win32":
    from eseas.core.refresh_excel import shut_excel, get_excel_app, refresh


class TestAskPermission:
    """Test ask_permission function"""

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_yes(self, mock_input):
        """Test permission granted with 'yes'"""
        mock_input.return_value = "yes"
        assert ask_permission() is True

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_y(self, mock_input):
        """Test permission granted with 'y'"""
        mock_input.return_value = "y"
        assert ask_permission() is True

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_ok(self, mock_input):
        """Test permission granted with 'ok'"""
        mock_input.return_value = "ok"
        assert ask_permission() is True

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_good(self, mock_input):
        """Test permission granted with 'good'"""
        mock_input.return_value = "good"
        assert ask_permission() is True

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_no(self, mock_input):
        """Test permission denied with 'no'"""
        mock_input.return_value = "no"
        assert ask_permission() is False

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_n(self, mock_input):
        """Test permission denied with 'n'"""
        mock_input.return_value = "n"
        assert ask_permission() is False

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_uppercase(self, mock_input):
        """Test permission with uppercase input"""
        mock_input.return_value = "YES"
        assert ask_permission() is True

    @patch("eseas.core.refresh_excel.get_input")
    def test_ask_permission_mixed_case(self, mock_input):
        """Test permission with mixed case input"""
        mock_input.return_value = "YeS"
        assert ask_permission() is True


class TestExcelRunning:
    """Test excel_running function"""

    @patch("eseas.core.refresh_excel.psutil.process_iter")
    def test_excel_not_running(self, mock_process_iter):
        """Test when Excel is not running"""
        mock_process_iter.return_value = [
            MagicMock(info={"name": "python.exe"}),
            MagicMock(info={"name": "chrome.exe"}),
        ]
        assert excel_running() is False

    @patch("eseas.core.refresh_excel.psutil.process_iter")
    def test_excel_running(self, mock_process_iter):
        """Test when Excel is running"""
        mock_process_iter.return_value = [
            MagicMock(info={"name": "python.exe"}),
            MagicMock(info={"name": "EXCEL.EXE"}),
        ]
        assert excel_running() is True

    @patch("eseas.core.refresh_excel.psutil.process_iter")
    def test_excel_running_lowercase(self, mock_process_iter):
        """Test when Excel process name is lowercase"""
        mock_process_iter.return_value = [
            MagicMock(info={"name": "excel.exe"}),
        ]
        assert excel_running() is True

    @patch("eseas.core.refresh_excel.psutil.process_iter")
    def test_excel_running_mixed_case(self, mock_process_iter):
        """Test when Excel process name is mixed case"""
        mock_process_iter.return_value = [
            MagicMock(info={"name": "Excel.exe"}),
        ]
        assert excel_running() is True

    @patch("eseas.core.refresh_excel.psutil.process_iter")
    def test_no_processes_running(self, mock_process_iter):
        """Test when no processes are running"""
        mock_process_iter.return_value = []
        assert excel_running() is False


class TestFileClass:
    """Test File class"""

    def test_file_init(self, tmp_path):
        """Test File initialization"""
        file_name = "test.xlsx"
        file = File(file_name, tmp_path)
        assert file.name == file_name
        assert file.folder == tmp_path
        assert isinstance(file.path, Path)

    def test_file_str_representation(self, tmp_path):
        """Test File string representation"""
        file = File("test.xlsx", tmp_path)
        str_repr = str(file)
        assert "<File [" in str_repr
        assert "]>" in str_repr
        assert "test.xlsx" in str_repr

    def test_file_exists_true(self, tmp_path):
        """Test File.exists() when file exists"""
        test_file = tmp_path / "test.xlsx"
        test_file.touch()
        file = File("test.xlsx", tmp_path)
        assert file.exists() is True

    def test_file_exists_false(self, tmp_path):
        """Test File.exists() when file doesn't exist"""
        file = File("nonexistent.xlsx", tmp_path)
        assert file.exists() is False

    def test_file_check_success(self, tmp_path):
        """Test File.check() when file exists"""
        test_file = tmp_path / "test.xlsx"
        test_file.touch()
        file = File("test.xlsx", tmp_path)
        # Should not raise exception
        file.check()

    def test_file_check_failure(self, tmp_path):
        """Test File.check() when file doesn't exist"""
        file = File("nonexistent.xlsx", tmp_path)
        with pytest.raises(ValueError, match="File Not found"):
            file.check()

    def test_file_path_is_absolute(self, tmp_path):
        """Test that File.path is absolute"""
        file = File("test.xlsx", tmp_path)
        assert file.path.is_absolute()

    def test_file_with_current_directory(self):
        """Test File with current directory"""
        file = File("test.xlsx", Path("."))
        assert isinstance(file.folder, Path)
        assert file.path.is_absolute()


@pytest.mark.skipif(sys.platform != "win32", reason="Windows-specific tests")
class TestWindowsSpecific:
    """Tests for Windows-specific functionality"""

    @patch("eseas.core.refresh_excel.subprocess.run")
    @patch("eseas.core.refresh_excel.ask_permission")
    def test_shut_excel_with_permission(self, mock_ask, mock_run):
        """Test shutting down Excel with user permission"""
        mock_ask.return_value = True
        mock_run.return_value = MagicMock(stdout="SUCCESS: Process terminated.")

        shut_excel(approve=False)

        mock_ask.assert_called_once()
        mock_run.assert_called_once()
        assert "taskkill" in mock_run.call_args[0][0]

    @patch("eseas.core.refresh_excel.subprocess.run")
    def test_shut_excel_auto_approve(self, mock_run):
        """Test shutting down Excel with auto-approve"""
        mock_run.return_value = MagicMock(stdout="SUCCESS: Process terminated.")

        shut_excel(approve=True)

        mock_run.assert_called_once()

    @patch("eseas.core.refresh_excel.ask_permission")
    def test_shut_excel_permission_denied(self, mock_ask):
        """Test shutting down Excel when permission is denied"""
        mock_ask.return_value = False

        with pytest.raises(ValueError, match="User did not confirm"):
            shut_excel(approve=False)

    @patch("eseas.core.refresh_excel.excel_running")
    @patch("eseas.core.refresh_excel.shut_excel")
    @patch("eseas.core.refresh_excel.win32com.client.Dispatch")
    def test_get_excel_app_excel_running(self, mock_dispatch, mock_shut, mock_running):
        """Test getting Excel app when Excel is running"""
        mock_running.return_value = True
        mock_excel = MagicMock()
        mock_dispatch.return_value = mock_excel

        result = get_excel_app(approve=True)

        mock_shut.assert_called_once_with(True)
        mock_dispatch.assert_called_once_with("Excel.Application")
        assert result == mock_excel

    @patch("eseas.core.refresh_excel.excel_running")
    @patch("eseas.core.refresh_excel.win32com.client.Dispatch")
    def test_get_excel_app_excel_not_running(self, mock_dispatch, mock_running):
        """Test getting Excel app when Excel is not running"""
        mock_running.return_value = False
        mock_excel = MagicMock()
        mock_dispatch.return_value = mock_excel

        result = get_excel_app(approve=False)

        mock_dispatch.assert_called_once_with("Excel.Application")
        assert result == mock_excel


@pytest.mark.skipif(sys.platform != "win32", reason="Windows-specific tests")
class TestFileRefresh:
    """Tests for File.refresh() method"""

    @patch("eseas.core.refresh_excel.get_excel_app")
    @patch("eseas.core.refresh_excel.time.sleep")
    def test_file_refresh_success(self, mock_sleep, mock_get_excel, tmp_path):
        """Test successful file refresh"""
        # Create a test file
        test_file = tmp_path / "test.xlsx"
        test_file.touch()

        # Mock Excel app
        mock_excel = MagicMock()
        mock_wb = MagicMock()
        mock_excel.Workbooks.Open.return_value = mock_wb
        mock_get_excel.return_value = mock_excel

        # Create File and refresh
        file = File("test.xlsx", tmp_path, approve=True)
        file.refresh()

        # Verify Excel operations
        mock_excel.Workbooks.Open.assert_called_once()
        mock_wb.RefreshAll.assert_called_once()
        mock_wb.Save.assert_called_once()
        mock_wb.Close.assert_called_once_with(False)

    @patch("eseas.core.refresh_excel.get_excel_app")
    def test_file_refresh_error_handling(self, mock_get_excel, tmp_path):
        """Test file refresh error handling"""
        # Create a test file
        test_file = tmp_path / "test.xlsx"
        test_file.touch()

        # Mock Excel app to raise exception
        mock_excel = MagicMock()
        mock_excel.Workbooks.Open.side_effect = Exception("Test error")
        mock_get_excel.return_value = mock_excel

        # Create File and refresh (should handle exception gracefully)
        file = File("test.xlsx", tmp_path, approve=True)
        # Should not raise exception
        file.refresh()


class TestCheckAndGetFiles:
    """Test check_and_get_files generator function"""

    def test_check_and_get_files_all_exist(self, tmp_path):
        """Test with all files existing"""
        # Create test files
        file1 = tmp_path / "file1.xlsx"
        file2 = tmp_path / "file2.xlsx"
        file1.touch()
        file2.touch()

        from eseas.core.refresh_excel import check_and_get_files

        files = ["file1.xlsx", "file2.xlsx"]
        result = list(check_and_get_files(files, tmp_path, approve=True))

        assert len(result) == 2
        assert all(isinstance(f, File) for f in result)

    def test_check_and_get_files_missing_file(self, tmp_path):
        """Test with missing file"""
        file1 = tmp_path / "file1.xlsx"
        file1.touch()

        from eseas.core.refresh_excel import check_and_get_files

        files = ["file1.xlsx", "missing.xlsx"]

        with pytest.raises(ValueError, match="File Not found"):
            list(check_and_get_files(files, tmp_path, approve=True))

    def test_check_and_get_files_empty_list(self, tmp_path):
        """Test with empty file list"""
        from eseas.core.refresh_excel import check_and_get_files

        result = list(check_and_get_files([], tmp_path, approve=True))
        assert len(result) == 0


@pytest.mark.skipif(sys.platform != "win32", reason="Windows-specific tests")
class TestRefreshFunction:
    """Tests for refresh() main function"""

    @patch("eseas.core.refresh_excel.pythoncom")
    @patch("eseas.core.refresh_excel.check_and_get_files")
    @patch("eseas.core.refresh_excel.excel_app", None)
    def test_refresh_function(self, mock_check_files, mock_com, tmp_path):
        """Test main refresh function"""
        # Create mock files
        mock_file1 = MagicMock(spec=File)
        mock_file2 = MagicMock(spec=File)
        mock_check_files.return_value = [mock_file1, mock_file2]

        files = ["file1.xlsx", "file2.xlsx"]
        refresh(files, tmp_path, approve=True)

        # Verify COM initialization
        mock_com.CoInitialize.assert_called_once()
        mock_com.CoUninitialize.assert_called_once()

        # Verify refresh called for each file
        mock_file1.refresh.assert_called_once()
        mock_file2.refresh.assert_called_once()

    @patch("eseas.core.refresh_excel.pythoncom")
    @patch("eseas.core.refresh_excel.check_and_get_files")
    def test_refresh_function_with_excel_app(self, mock_check_files, mock_com, tmp_path):
        """Test refresh function with existing excel_app"""
        import eseas.core.refresh_excel as refresh_module

        # Mock excel app
        mock_excel = MagicMock()
        refresh_module.excel_app = mock_excel

        mock_file = MagicMock(spec=File)
        mock_check_files.return_value = [mock_file]

        try:
            refresh(["file1.xlsx"], tmp_path, approve=True)

            # Verify Excel quit was called
            mock_excel.Quit.assert_called_once()
        finally:
            # Clean up
            refresh_module.excel_app = None
